!*********************************************
!*
!* Module containing the subroutine computing 
!* the Discrete Fourier Transform
!*
!*********************************************

MODULE dft_module

  !* module loaded

  USE constants

  contains

    !*****************************************
    !*
    !* Subroutine computing the Discrete
    !* Fourier Transform
    !*
    !* INPUT
    !*
    !* x :: The initial vector
    !*
    !* OUTPUT
    !*
    !* y :: The DFT of the input vector
    !*
    !* INPUT
    !*
    !* sens :: integer that should be equal
    !* to FORWARD or BACKWARD
    !*
    !*****************************************

    SUBROUTINE dft(x,y,sens)

      !* Variable declarations

      REAL(idp) :: pi
      INTEGER :: sens
      COMPLEX(idp), DIMENSION(:) :: x,y
      COMPLEX(idp), DIMENSION(size(x,1), size(x,1)) :: dft_matrix
      COMPLEX(idp) :: W
      COMPLEX(idp) :: i

      INTEGER :: n
      INTEGER :: j,k

      !* Code

      !** Some constants for the computation

      n=size(x,1)

      pi=4.d0*atan(1.d0)

      i=CMPLX(0.d0,1.d0,idp)

      if(sens==FORWARD) then
         W=exp(-2.d0*i*pi/REAL(n,idp))
      else
         W=exp(2.d0*i*pi/REAL(n,idp))
      end if

      !** Computation of the DFT matrix

      do j=1,n
         do k=1,n
            dft_matrix(j,k)=(1.d0/sqrt(REAL(n,idp)))*W**((j-1)*(k-1))
         end do
      end do

      !* Matrix-Vector product

      call zgemv('n',n,n,1.d0,dft_matrix,n,x,1,0.d0,y,1)

    END SUBROUTINE dft

  END MODULE dft_module
