!**********************************************
!*
!* Module contianing the function creating
!* the radix components for the fft
!*
!* exemple : for n=6 and radix=2
!*
!* x = (0 1 2 3 4 5)
!* y = (0 2 4 1 3 5)
!*
!*********************************************

MODULE utility

  !* module loaded
  USE constants

  contains

    !****************************************
    !*
    !* Subroutine for the computation of the
    !* radix components for the fft
    !*
    !* INPUT
    !*
    !* x :: vector to rearrange
    !*
    !* OUTPUT
    !*
    !* y :: vector rearranged for the 
    !*      corresponding radix
    !*
    !* INPUT
    !*
    !* radix :: radix
    !*
    !****************************************
    SUBROUTINE get_radix_components(x,y,radix)

      !* Variable declarations

      COMPLEX(idp), DIMENSION(:) :: x
      COMPLEX(idp), DIMENSION(:) :: y

      INTEGER :: radix

      INTEGER :: i,j,k

      INTEGER :: new_size

      !* Code
      
      !** size of the radix parts of the output vector

      new_size=size(x,1)/radix

      do k=0,radix-1
         j=1
         do i=1,size(x,1)
            if(modulo(i-1,radix)==k) then
               y(j+k*new_size)=x(i)
               j=j+1
            end if
         end do
      end do

    end SUBROUTINE get_radix_components

  end MODULE utility
